@@ -12,15 +12,20 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from contextlib import contextmanager
 import datetime
 import functools
+import os
 from os import path
 import time
 import types
 import uuid
 
 import mock
+from oslo_config import cfg
 import oslotest.base as oslotest
+from oslotest import createfile
+
 import six
 from six.moves.urllib import parse
 import webtest
@@ -28,10 +33,30 @@ import webtest
 from OpenSSL import crypto
 
 from barbican.api import app
+from barbican.common import config
 import barbican.context
+from barbican.model import repositories
+from barbican.plugin.crypto import manager as cm
+from barbican.plugin.crypto import p11_crypto
+
+from barbican.plugin.interface import secret_store
+from barbican.plugin import kmip_secret_store as kss
 from barbican.tests import database_utils
 
 
+def mock_pecan_request(test_instance, host=None):
+    patcher_obj = mock.patch('pecan.request')
+    mock_req = patcher_obj.start()
+    test_instance.addCleanup(patcher_obj.stop)
+    mock_req.url = host
+
+
+@contextmanager
+def pecan_context(test_instance, host=None):
+    mock_pecan_request(test_instance, host=host)
+    yield
+
+
 class BarbicanAPIBaseTestCase(oslotest.BaseTestCase):
     """Base TestCase for all tests needing to interact with a Barbican app."""
     root_controller = None
@@ -81,6 +106,9 @@ class BaseTestCase(oslotest.BaseTestCase):
 
     def tearDown(self):
         super(BaseTestCase, self).tearDown()
+        ss_conf = config.get_module_config('secretstore')
+        ss_conf.clear_override("enable_multiple_secret_stores",
+                               group='secretstore')
 
 
 class MockModelRepositoryMixin(object):
@@ -274,6 +302,32 @@ class MockModelRepositoryMixin(object):
             mock_repo_obj=mock_preferred_ca_repo,
             patcher_obj=self.mock_preferred_ca_repo_patcher)
 
+    def setup_secret_stores_repository_mock(
+            self, mock_secret_stores_repo=mock.MagicMock()):
+        """Mocks the project repository factory function
+
+        :param mock_secret_stores_repo: The pre-configured mock secret stores
+                                     repo to be returned.
+        """
+        self.mock_secret_stores_repo_patcher = None
+        self._setup_repository_mock(
+            repo_factory='get_secret_stores_repository',
+            mock_repo_obj=mock_secret_stores_repo,
+            patcher_obj=self.mock_secret_stores_repo_patcher)
+
+    def setup_project_secret_store_repository_mock(
+            self, mock_project_secret_store_repo=mock.MagicMock()):
+        """Mocks the project repository factory function
+
+        :param mock_project_secret_store_repo: The pre-configured mock project
+                                     secret store repo to be returned.
+        """
+        self.mock_proj_secret_store_repo_patcher = None
+        self._setup_repository_mock(
+            repo_factory='get_project_secret_store_repository',
+            mock_repo_obj=mock_project_secret_store_repo,
+            patcher_obj=self.mock_proj_secret_store_repo_patcher)
+
     def setup_project_ca_repository_mock(
             self, mock_project_ca_repo=mock.MagicMock()):
         """Mocks the project repository factory function
@@ -309,7 +363,8 @@ def construct_new_test_function(original_func, name, build_params):
         six.get_function_code(original_func),
         six.get_function_globals(original_func),
         name=name,
-        argdefs=six.get_function_defaults(original_func)
+        argdefs=six.get_function_defaults(original_func),
+        closure=six.get_function_closure(original_func)
     )
 
     for key, val in original_func.__dict__.items():
@@ -384,6 +439,142 @@ def parameterized_dataset(build_data):
     return decorator
 
 
+def setup_oslo_config_conf(testcase, content, conf_instance=None):
+
+    conf_file_fixture = testcase.useFixture(
+        createfile.CreateFileWithContent('barbican', content))
+    if conf_instance is None:
+        conf_instance = cfg.CONF
+    conf_instance([], project="barbican",
+                  default_config_files=[conf_file_fixture.path])
+
+    testcase.addCleanup(conf_instance.reset)
+
+
+def setup_multiple_secret_store_plugins_conf(testcase, store_plugin_names,
+                                             crypto_plugin_names,
+                                             global_default_index,
+                                             conf_instance=None,
+                                             multiple_support_enabled=None):
+    """Sets multiple secret store support conf as oslo conf file.
+
+    Generating file based conf based on input store and crypto plugin names
+    provided as list. Index specified in argument is used to mark that specific
+    secret store as global_default = True.
+
+    Input lists are
+    'store_plugins': ['store_crypto', 'kmip_plugin', 'store_crypto'],
+    'crypto_plugins': ['simple_crypto', '', 'p11_crypto'],
+
+    Sample output conf file generated is
+
+    [secretstore]
+    enable_multiple_secret_stores = True
+    stores_lookup_suffix = plugin_0, plugin_1, plugin_2
+
+    [secretstore:plugin_0]
+    secret_store_plugin = store_crypto
+    crypto_plugin = simple_crypto
+    global_default = True
+
+    [secretstore:plugin_1]
+    secret_store_plugin = kmip_plugin
+
+    [secretstore:plugin_2]
+    secret_store_plugin = store_crypto
+    crypto_plugin = p11_crypto
+
+    """
+
+    def _get_conf_line(name, value, section=None):
+        out_line = "\n[{0}]\n".format(section) if section else ""
+        out_line += "{0} = {1}\n".format(name, value) if name else ""
+        return out_line
+
+    if multiple_support_enabled is None:
+        multiple_support_enabled = True
+
+    conf_content = ""
+
+    if store_plugin_names is not None:
+
+        if len(store_plugin_names) < len(crypto_plugin_names):
+            max_count = len(crypto_plugin_names)
+        else:
+            max_count = len(store_plugin_names)
+
+        lookup_names = ['plugin_{0}'.format(indx) for indx in range(max_count)]
+        section_names = ['secretstore:{0}'.format(lname) for lname in
+                         lookup_names]
+        lookup_str = ", ".join(lookup_names)
+
+        conf_content = _get_conf_line('enable_multiple_secret_stores',
+                                      multiple_support_enabled,
+                                      section='secretstore')
+        conf_content += _get_conf_line('stores_lookup_suffix',
+                                       lookup_str, section=None)
+
+        for indx, section_name in enumerate(section_names):
+            if indx < len(store_plugin_names):
+                store_plugin = store_plugin_names[indx]
+                conf_content += _get_conf_line('secret_store_plugin',
+                                               store_plugin,
+                                               section=section_name)
+            else:
+                conf_content += _get_conf_line(None, None,
+                                               section=section_name)
+            if indx < len(crypto_plugin_names):
+                crypto_plugin = crypto_plugin_names[indx]
+                conf_content += _get_conf_line('crypto_plugin', crypto_plugin,
+                                               section=None)
+            if indx == global_default_index:
+                conf_content += _get_conf_line('global_default', 'True',
+                                               section=None)
+
+    setup_oslo_config_conf(testcase, conf_content, conf_instance)
+
+
+class MultipleBackendsTestCase(database_utils.RepositoryTestCase):
+
+    def setUp(self):
+        super(MultipleBackendsTestCase, self).setUp()
+
+    def _mock_plugin_settings(self):
+
+        kmip_conf = kss.CONF
+        kmip_conf.kmip_plugin.username = "sample_username"
+        kmip_conf.kmip_plugin.password = "sample_password"
+        kmip_conf.kmip_plugin.keyfile = None
+        kmip_conf.kmip_plugin.pkcs1_only = False
+
+        pkcs11_conf = p11_crypto.CONF
+        pkcs11_conf.p11_crypto_plugin.library_path = "/tmp"  # any dummy path
+
+    def init_via_conf_file(self, store_plugin_names, crypto_plugin_names,
+                           enabled=True, global_default_index=0):
+        secretstore_conf = config.get_module_config('secretstore')
+
+        setup_multiple_secret_store_plugins_conf(
+            self, store_plugin_names=store_plugin_names,
+            crypto_plugin_names=crypto_plugin_names,
+            global_default_index=global_default_index,
+            conf_instance=secretstore_conf,
+            multiple_support_enabled=enabled)
+
+        # clear globals if already set in previous tests
+        secret_store._SECRET_STORE = None  # clear secret store manager
+        cm._PLUGIN_MANAGER = None  # clear crypto manager
+        self._mock_plugin_settings()
+
+    def _get_secret_store_entry(self, store_plugin, crypto_plugin):
+        all_ss = repositories.get_secret_stores_repository().get_all()
+        for ss in all_ss:
+            if (ss.store_plugin == store_plugin and
+                    ss.crypto_plugin == crypto_plugin):
+                return ss
+        return None
+
+
 def create_timestamp_w_tz_and_offset(timezone=None, days=0, hours=0, minutes=0,
                                      seconds=0):
     """Creates a timestamp with a timezone and offset in days
@@ -450,12 +641,12 @@ def generate_test_uuid(tail_value=0):
 
 
 def get_symmetric_key():
-    s = "MIICdgIBADANBgkqhkiG9w=="
+    s = b"MIICdgIBADANBgkqhkiG9w=="
     return s
 
 
 def get_triple_des_key():
-    s = "AQIDBAUGBwgBAgMEBQYHCAECAwQFBgcI"
+    s = b"AQIDBAUGBwgBAgMEBQYHCAECAwQFBgcI"
     return s
 
 
@@ -478,5 +669,9 @@ def is_public_key_valid(expected, observed):
     return True
 
 
+def is_kmip_enabled():
+    return os.environ.get('KMIP_PLUGIN_ENABLED') is not None
+
+
 class DummyClassForTesting(object):
     pass
